from cron_migration.app import manager as app_manager
from cron_migration.files.managers import files as files_manager
from cron_migration.files.models.path import Path
from cron_migration.files.managers.path import PathManager
from cron_migration.app.models.environment import Environment
from cron_migration.app import exit_codes
from cron_migration.cli.models.command import BaseCommand
from cron_migration.cli.manager import CommandsManager


@CommandsManager.bind("init")
class Init(BaseCommand):
    def __init__(self, env: Environment):
        self._env = env
        self._path_manager = PathManager()
        self._path_manager.set_path(self._env._path)
        self._template = None

    @BaseCommand._output.printed_task(
        BaseCommand._output.blue,
        "Checking if path is already exist",
        BaseCommand._output.header,
        "Done.",
        BaseCommand._output.red,
        "Path is already exists",
        None
    )
    def _path_exists(self):
        if self._path_manager.is_dir():
            return exit_codes.FOLDER_ALREADY_EXISTS
        return None

    @BaseCommand._output.printed_task(
        BaseCommand._output.blue,
        "looking for a template",
        BaseCommand._output.header,
        "Done.",
        BaseCommand._output.red,
        "Template is not exists",
        str()
    )
    def _get_template(self):
        return app_manager.get_template(self._env.template)

    @BaseCommand._output.printed_task(
        BaseCommand._output.blue,
        "Copying files...",
        BaseCommand._output.header,
        "Done.",
        success_indicator=None
    )
    def _copy_files(self):
        if isinstance(err := files_manager.copy_tree(Path(self._template), self._env._path), int):
            return err
        return None

    @BaseCommand._output.printed_task(
        BaseCommand._output.blue,
        "can not access, trying to remove",
        BaseCommand._output.header,
        "removed successfully",
        BaseCommand._output.red,
        "Could not remove tree, please remove it manually",
        None
    )
    def _remove_files(self):
        if (self._path_manager.remove_tree()) is not True:
            return exit_codes.COULD_NOT_REMOVE_BASE_FOLDER
        return None

    @BaseCommand._output.printed_task(
        BaseCommand._output.blue,
        "Validate environment access",
        on_fail_color=BaseCommand._output.red,
        on_fail_msg="Could not remove tree, please remove it manually",
        success_indicator=True
    )
    def _validate_access(self):
        if not self._path_manager.has_access():
            if error := self._remove_files():
                return error
            return exit_codes.PATH_NO_ACCESS
        return True

    @BaseCommand._output.printed_task(
        BaseCommand._output.blue,
        "Preparing environment.....",
        BaseCommand._output.green,
        "Done.",
        BaseCommand._output.red,
        "Failed!",
        success_indicator=0
    )
    def run(self):
        if error := self._path_exists():
            return error

        if not isinstance(template := self._get_template(), str):
            return template
        self._template = template
        self._copy_files()
        if not isinstance(has_access := self._validate_access(), bool):
            return has_access

        return 0



from abc import ABCMeta, abstractmethod


class SimpleProxyMeta(type, metaclass=ABCMeta):
    _instance = None

    @abstractmethod
    def _fwd_object(cls):
        """
        should returns a target object
        :return:
        """

    def _get_object(cls):
        if cls._instance is None:
            cls._instance = cls._fwd_object()
        return cls._instance

    def __getattr__(cls, item):
        def wrapper(*args, **kwargs):
            return getattr(cls._get_object(), item)(*args, **kwargs)

        return wrapper


import subprocess


class Driver:
    def __init__(self):
        self._options = {}
        self._const_options = {}

    def _exec(self):
        options = [arg for k, v in self._const_options.items() for arg in ([k]+[v] if v else [k])]
        options += [arg for k, v in self._options.items() for arg in ([k]+[v] if v else [k])]
        self._options = {}
        process = subprocess.Popen(["crontab"] + options, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        exit_code = process.wait()
        result = (exit_code, process.stdout, process.stderr)
        return result

    def add_option(self, key, value=""):
        self._options[key] = value

    def add_const_option(self, key, value=""):
        self._const_options[key] = value

    def all_jobs(self):
        self.add_option("-l")
        return self._exec()

    def user(self, u):
        self.add_const_option("-u", u)

    def delete(self):
        self.add_option("-r")
        return self._exec()

    def insert(self, filepath):
        self.add_option(filepath)
        self._exec()



import os
import shutil
from cron_migration.files.models.path import Path
from cron_migration.app import exit_codes


def copy_tree(src: Path, dest: Path):
    try:
        shutil.copytree(src.path, dest.path)
    except Exception as e:
        return exit_codes.COULD_NOT_COPY_FILES


def mkdir(path: Path):
    try:
        os.mkdir(path.path)
    except Exception:
        return False
    return True



from cron_migration.app.models.environment import Environment
import os
import importlib.util
import json
from typing import Iterator, Set, Dict
from cron_migration.revisions.manager import TaskManager


class RevisionMapper:
    def __init__(self, environment: Environment):
        self._environment = environment
        self.revisions: Dict[str, TaskManager] = {}
        self.latest: Set[str] = set()
        self._last_upgraded_revision = ""
        self._first_revision: TaskManager = None
        self._heads: Dict[str, TaskManager] = {}
        self._tails: Dict[str, TaskManager] = {}

    def stream_revisions(self) -> Iterator[TaskManager]:
        revision_path = self._environment.get_revisions_path()
        for module_ in os.listdir(revision_path.path):
            if "__pycache__" in module_:
                continue

            module_path = revision_path.join(module_)
            spec = importlib.util.spec_from_file_location(module_, f"{module_path}")
            revision = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(revision)
            yield TaskManager(revision.Revision, module_path)

    def review(self):
        for revision in self.stream_revisions():
            if revision.get_revision_id() in self.revisions:
                # todo: handle error: duplicate
                continue
            self.revisions[revision.get_revision_id()] = revision
            if revision.get_down_revision() is None:
                self._first_revision = revision
                self._tails[revision.get_revision_id()] = revision
            self.latest.add(revision.get_revision_id())

        for revision in self.revisions.values():
            if revision.get_down_revision() is None:
                continue
            prev_revision = self.revisions[revision.get_down_revision()]
            prev_revision.set_next(revision.task)
            if prev_revision.get_revision_id() in self.latest:
                self.latest.discard(prev_revision.get_revision_id())

        for revision_id in self._tails:
            tail = revision_id
            revision = self.revisions[revision_id]
            while revision:
                revision_id = revision.get_revision_id()
                revision = self.revisions[revision.get_revision_id()].next

            self._tails[tail] = self.revisions[revision_id]  # start to end
            self._heads[revision_id] = self.revisions[tail]  # end to start

    def total_heads(self):
        return len(self._heads)

    def get_latest_revision(self):
        for revision_id in self.latest:
            return revision_id
        return None

    @property
    def get_revision_to_upgrade(self):
        with open(self._environment.path_from_base('.rvsn'), "r") as f:
            json_ = {}
            for tail in self._tails:
                try:
                    json_ = json.load(f) if not json_ else json_
                except json.JSONDecodeError:
                    ...
                try:
                    # todo handle error
                    revision_key = tail
                    if revision_line := json_.get(tail, None):
                        if not self.revisions[revision_line].next:
                            raise Exception()
                        revision_key = self.revisions[revision_line].next.get_revision_id()

                    yield revision_key
                except (FileNotFoundError, Exception):
                    continue

    def get_tail_from_head(self, revision: TaskManager):
        return self._heads[revision.get_revision_id()].get_revision_id()

    def get_head_from_tail(self, revision: TaskManager):
        return self._tails[revision.get_revision_id()].get_revision_id()

    def _get_first_revision(self):
        return self._first_revision

    def get_waiting_list(self):
        for revision_to_update in self.get_revision_to_upgrade:
            revision = self.revisions[revision_to_update]
            while revision:
                yield (self.revisions[revision.get_revision_id()])
                revision = self.revisions[revision.get_revision_id()].next



from abc import abstractmethod


class Task:
    @abstractmethod
    def upgrade(self):
        ...

    @abstractmethod
    def downgrade(self):
        ...

    @classmethod
    @abstractmethod
    def get_revision_id(cls) -> str:
        ...

    @classmethod
    @abstractmethod
    def get_prev_revision_id(cls) -> str:
        ...



import click
import os
from cron_migration.cli import environment
from cron_migration.files.models.path import Path
from cron_migration.cli import OperationsFacade
from cron_migration.revisions.model import Revision
from cron_migration.revisions.services.new_revision import NewRevisionService
from cron_migration.revisions.services.revision_apply import RevisionApply
from cron_migration.revisions.services.mapper import RevisionMapper


@click.group()
def cronmig():
    ...

@cronmig.command()
@click.option('-n', '--dir-name', default="cronjobs")
def init(dir_name):
    """Creates a new envrionment"""
    environment.path = Path(os.path.join(os.getcwd(), dir_name))
    exit(int(OperationsFacade.init(environment)))


@click.group()
def revision():
    ...


@revision.command()
@click.option('-n', '--dir-name', default="cronjobs")
@click.argument("message")
def make(message, dir_name):
    environment.path = Path(os.path.join(os.getcwd(), dir_name))
    revision_service = NewRevisionService(Revision(), RevisionMapper(environment), environment)
    exit(int(OperationsFacade.make_revision(revision_service, message)))


@revision.command()
@click.option('-n', '--dir-name', default="cronjobs")
def upgrade(dir_name):
    environment.path = Path(os.path.join(os.getcwd(), dir_name))
    service = RevisionApply(RevisionMapper(environment), environment)
    exit(int(OperationsFacade.upgrade(service)))


@revision.command()
@click.argument("steps")
@click.option('-n', '--dir-name', default="cronjobs")
def downgrade(steps, dir_name):
    environment.path = Path(os.path.join(os.getcwd(), dir_name))
    service = RevisionApply(RevisionMapper(environment), environment)
    exit(int(OperationsFacade.downgrade(service, steps)))


if __name__ == '__main__':
    ...
    # init()
    #revision()



from cron_migration.cli.models.command import BaseCommand
from cron_migration.cli.manager import CommandsManager
from cron_migration.revisions.services.revision_apply import RevisionApply


@CommandsManager.bind("upgrade")
class Upgrade(BaseCommand):
    def __init__(self, service: RevisionApply):
        self._service = service

    def run(self):
        for revision in self._service.get_upgrades_list():
            log = BaseCommand._output.printed_task(
                BaseCommand._output.blue,
                f"upgrading file: {revision.path}, revision id: {revision.get_revision_id()}",
                BaseCommand._output.green,
                "Done",
                BaseCommand._output.red,
                "Failed!",
                success_indicator=None
            )(lambda: self._service.upgrade(revision))
            log()

        return 0



from .mapper import RevisionMapper
from cron_migration.revisions.manager import TaskManager
from cron_migration.app.models.environment import Environment
import json


class RevisionApply:
    def __init__(self, mapper: RevisionMapper, environment: Environment):
        self._mapper = mapper
        self._environment = environment
        self._mapper.review()

    def _save_last_revision(self, revision: TaskManager, tail: str):
        with open(self._environment.path_from_base('.rvsn'), "r+") as f:
            try:
                json_ = json.load(f)
            except json.JSONDecodeError:
                json_ = {}
            json_[tail] = revision.get_revision_id() if revision is not None else ""
            f.seek(0)
            f.truncate()
            json.dump(json_, f)

    def upgrade(self, revision: TaskManager):
        revision.upgrade()
        if not revision.next:
            self._save_last_revision(revision, self._mapper.get_tail_from_head(revision))

    def downgrade(self, revision: TaskManager, tail: str):
        revision.downgrade()
        self._save_last_revision(self._mapper.revisions.get(revision.get_down_revision(), None), tail)

    def get_downgrades_list(self, steps):
        revision_signature = self._mapper.get_latest_revision()
        if self._mapper.total_heads() > 1:
            raise Exception("Please Merge files before downgrading....")
        for _ in range(int(steps), 0, -1):
            if not revision_signature:
                break
            revision = self._mapper.revisions[revision_signature]
            yield revision
            revision_signature = revision.get_down_revision()

    def get_upgrades_list(self):
        return self._mapper.get_waiting_list()

    def get_tail_from_head(self, revision: TaskManager):
        return self._mapper.get_tail_from_head(revision)



from cron_migration.system.meta.singleton import Singleton
from cron_migration.files.models.path import Path
import os
import shutil


class PathManager(metaclass=Singleton):

    def __init__(self):
        self.path: Path = None

    def set_path(self, path: Path):
        self.path = path

    def is_dir(self):
        return os.path.isdir(self.path.path)

    def has_access(self):
        return os.access(self.path.path, os.F_OK)

    def remove_tree(self):
        try:
            shutil.rmtree(self.path.path)
        except Exception:
            return False



from cron_migration.revisions.model import Revision
from .mapper import RevisionMapper
import uuid
import datetime
from mako.template import Template


class NewRevisionService:
    def __init__(self, revision: Revision, revision_map: 'RevisionMapper', environment: 'Environment'):
        self._revision = revision
        self._mapper = revision_map
        self._environment = environment

    def _set_revision_signature(self, sig=None):
        self._revision.signature = uuid.uuid4().hex.strip("-")[-16:] if sig is None else sig
        return self._revision.signature

    def _set_date(self, dt_format=""):
        self._revision.date = datetime.datetime.now().strftime(dt_format if dt_format else "%d/%m/%Y %H:%M:%S")
        return self._revision.date

    def _set_message(self, message):
        self._revision.message = message
        return self._revision.message

    def _set_down_revision(self, down_revision):
        self._revision.down_revision = down_revision

    def get(self):
        return self._revision

    def _review(self):
        self._mapper.review()

    def _make_template_file(self):
        revision = self.get()
        template = Template(filename=u"{}".format(self._environment.get_script_path().path))
        dst = u"{}_{}.py".format(revision.signature, revision.message)
        with open(self._environment.get_revisions_path(dst).path, "wb") as f:
            f.write(
                template.render_unicode(
                    revision=revision.signature,
                    prev_revision=revision.down_revision,
                    date=revision.date,
                    message=revision.message
                ).encode("utf-8")
            )
        return self._environment.get_revisions_path(dst).path

    def make_revision_file(self, message):
        self._review()
        self._set_revision_signature()
        self._set_date()
        self._set_message(message)
        self._set_down_revision(self._mapper.get_latest_revision())
        return self._make_template_file()



from abc import ABCMeta, abstractmethod
from cron_migration.cli.output import Output
from .commandError import CommandError


class Command(metaclass=ABCMeta):
    @abstractmethod
    def run(self):
        ...


class BaseCommand(Command):
    _output = Output()

    @abstractmethod
    def run(self):
        pass

    @staticmethod
    def fail(error_code=None):
        raise CommandError(error_code=error_code)



import os


class Revision:
    def __init__(self, **kwargs):
        self.signature: str = kwargs.get("signature", "")
        self._message: str = kwargs.get("message", "")
        self.date = kwargs.get("date", "")
        self.down_revision = None

    def __str__(self):
        return f"{self.signature}\t{self.message}\t{self.date}"

    @property
    def message(self):
        return os.path.normpath(self._message)

    @message.setter
    def message(self, m: str):
        self._message = m.replace(" ", "_")



from cron_migration.cli.models.command import BaseCommand
from cron_migration.cli.manager import CommandsManager
from cron_migration.revisions.services.revision_apply import RevisionApply


@CommandsManager.bind("downgrade")
class Downgrade(BaseCommand):
    def __init__(self, service: RevisionApply, steps: int):
        self._service = service
        self._steps = steps

    def run(self):
        for revision in self._service.get_downgrades_list(self._steps):
            try:
                tail
            except:
                tail = self._service.get_tail_from_head(revision)

            log = BaseCommand._output.printed_task(
                BaseCommand._output.blue,
                f"downgrading file: {revision.path}, revision id: {revision.get_revision_id()}",
                BaseCommand._output.green,
                "Done",
                BaseCommand._output.red,
                "Failed!",
                success_indicator=None
            )(lambda: self._service.downgrade(revision, tail))
            log()

        return 0



from .task import Task
from typing import Type


class TaskManager:
    def __init__(self, task: Type[Task], path):
        self.task = task
        self._all_down_revisions = set()
        self.next: Task = None
        self.prev: Task = None
        self.path = path

    def get_down_revision(self):
        return self.task.get_prev_revision_id()

    def get_revision_id(self):
        return self.task.get_revision_id()

    def set_next(self, task: Task):
        self.next = task

    def set_prev(self, task: Task):
        self.prev = task

    def upgrade(self):
        self.task().upgrade()

    def downgrade(self):
        self.task().downgrade()



from cron_migration.conf import templates
from cron_migration.files.managers.path import PathManager
from cron_migration.files.models.path import Path
from cron_migration.app import exit_codes
import os


def get_templates_path():
    return templates.path


def get_template(template):
    path = Path(os.path.join(get_templates_path(), template))
    path_manager = PathManager()
    path_manager.set_path(path)
    if not path_manager.is_dir():
        return exit_codes.INVALID_TEMPLATE
    return path.path



from cron_migration.revisions.services.new_revision import NewRevisionService
from cron_migration.cli.models.command import BaseCommand
from cron_migration.cli.manager import CommandsManager


@CommandsManager.bind("make_revision")
class MakeRevision(BaseCommand):
    def __init__(self, _new_revision_service: NewRevisionService, message):
        self._new_revision_service = _new_revision_service
        self._message = message

    def run(self):
        filename = self._new_revision_service.make_revision_file(self._message)
        log = BaseCommand._output.printed_task(
            BaseCommand._output.blue,
            "generating revision file...",
            BaseCommand._output.green,
            filename,
            BaseCommand._output.red,
            "Failed!",
            success_indicator=str
        )(lambda: filename)
        log()

        return 0



from cron_migration.files.models.path import Path
import os


class Environment:
    def __init__(self, path: Path = None, template: str = "default"):
        self._path = path
        self.template = template

    def get_real_path(self):
        return self.path.path

    def get_script_path(self) -> Path:
        return Path(
            path=os.path.join(
                self.get_real_path() +
                os.path.sep +
                "revision.py.mako"
            )
        )

    def path_from_base(self, *args):
        return self._path.join(*args)

    def get_revisions_path(self, filename=None) -> Path:
        return Path(
            path=os.path.join(
                self.get_real_path() +
                os.path.sep +
                "revisions" +
                (os.path.sep + filename if filename else "")

            )
        )

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, p):
        self._path = p



import os
from .job import Job
from .scheduler import Scheduler
from .driver import Driver
from .utils import LazyProperty


class Crontab:

    def __init__(self, user=None, scheduler: Scheduler = None):
        self._driver = Driver()
        self._scheduler = scheduler
        if user:
            self._driver.user(user)

    def new_job(self, *args, **kwargs) -> Job:
        return Job(*args, **kwargs)

    @LazyProperty
    def all_jobs(self) -> set:
        exist_code, stdout, stderr = self._driver.all_jobs()
        commands = set()
        while (line := stdout.readline()):
            line = line.decode("utf-8")
            commands.add(line)
        return commands

    def insert(self, job: Job):
        commands = self.all_jobs
        commands.add(job.__str__())
        self._save_changes(commands)

    def remove(self, job: Job):
        commands = self.all_jobs
        commands.discard(job.__str__())
        self._save_changes(commands)

    def _save_changes(self, commands):
        path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "tab.txt")

        try:
            os.unlink(path)
        except:
            ...
        with open(path, "a") as f:
            for command in commands:
                f.write(command)
        self._driver.insert(path)
        os.unlink(path)




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="itay-bardugo-cronmig", # Replace with your own username
    version="1.0.0",
    author="Itay Bardugo",
    author_email="itaybardugo91@gmail.com",
    description="manage your cron jobs with python and a revision system(CLI)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
    ],
    entry_points={
        "console_scripts": [
            'cronmig=cron_migration.cli.commands:cronmig',
            'cronmig-revision=cron_migration.cli.commands:revision',
        ]
    },
    python_requires='>=3.8.3',
)



from .manager import CommandsManager
from cron_migration.system.meta.proxies import SimpleProxyMeta


class OperationProxy(SimpleProxyMeta):

    def _fwd_object(cls):
        return CommandsManager()


class OperationsFacade(metaclass=OperationProxy):
    """
    A proxy class to OpeartaionManager
    """



import os
from datetime import datetime


class Output:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    RED = '\033[31m'
    RESET = '\033[0m'
    __one_line = False
    __with_date = True

    @staticmethod
    def red(msg):
        __class__._print(msg, __class__.RED)

    @staticmethod
    def blue(msg):
        __class__._print(msg, __class__.OKBLUE)

    @staticmethod
    def header(msg):
        __class__._print(msg, __class__.HEADER)

    @staticmethod
    def green(msg):
        __class__._print(msg, __class__.OKGREEN)

    @staticmethod
    def _print(output, color=""):
        msg = []
        if __class__.__with_date:
            msg.extend([os.linesep, datetime.now().strftime('%Y-%m-%d %H:%M:%S')])
        msg.extend([color, output, __class__.RESET])
        print(*msg, end=" " if Output.__one_line else os.linesep)

    @staticmethod
    def printed_task(pre_print: callable = "", pre_msg: callable = "", on_success_color: callable = "",
                     on_success_msg: callable = "", on_fail_color: callable = "",
                     on_fail_msg="", success_indicator: type = ""):

        def wrapper(fn):
            def func(*args, **kwargs):
                Output.__one_line = True
                if pre_print:
                    pre_print(pre_msg.title())
                    Output.__with_date = False

                res = fn(*args, **kwargs)
                if success_indicator != "":
                    has_success_match = isinstance(success_indicator, type(res)) or type(res) is success_indicator
                    if on_success_color and has_success_match:
                        on_success_color(on_success_msg.title())
                        Output.__with_date = False
                    elif on_fail_color and not has_success_match:
                        on_fail_color(on_fail_msg.title())
                Output.__one_line = False
                Output.__with_date = True
                return res

            return func

        return wrapper



from cron_migration.cli.models.command import Command
from typing import Dict, Union, Callable


class CommandsManager:
    __commands: Dict[str, Union[Callable[..., Command], Command]] = {}

    def __init__(self):
        self._request = ""

    def __getattr__(self, item):
        self._request = item

        def wrapper(*args, **kwargs):
            return self._get_app(*args, **kwargs).run()

        return wrapper

    def _get_app(self, *args, **kwargs):
        if self._instance_exists():
            return
        self._cache_app(*args, **kwargs)
        return self._get_command()

    def _cache_app(self, *args, **kwargs):
        CommandsManager.__commands[self._request] = self._get_command()(*args, **kwargs)

    def _get_command(self):
        return CommandsManager.__commands[self._request]

    def _instance_exists(self):
        return True if isinstance(self._get_command(), Command) else False

    @staticmethod
    def bind(alias: str):
        assert alias not in CommandsManager.__commands, "alias '{}' already exists".format(alias)

        def wraper(cls: Callable[..., Command]):
            def callback(*args, **kwargs):
                return cls(*args, **kwargs)

            CommandsManager.__commands[alias] = callback

        return wraper



import os
from .scheduler import Scheduler


class Job:
    def __init__(self, command, minute="*", hour="*", month_day="*", month="*", week_day="*"):
        self._timing = [minute, hour, month_day, month, week_day]
        self.command = command + os.linesep
        self._scheduler = None

    @property
    def timing(self):
        return " ".join(self.scheduler.time if self.scheduler is not None else self._timing)

    def set_timing(self, t: list):
        """
        :param t: a list with 5 cells  [minute, hour, month_day, month, week_day]
        :return:
        """
        length = len(t)
        if length > 5:
            raise Exception("timing format: [minute, hour, month_day, month, week_day]")
        self._timing[length:5] = ["*"] * (5 - length)

    def set_minute(self, m):
        self._timing[0] = m

    def set_hour(self, h):
        self._timing[1] = h

    def set_month_day(self, md):
        self._timing[2] = md

    def set_month(self, m):
        self._timing[3] = m

    def set_week_day(self, wd):
        self._timing[4] = wd

    @property
    def scheduler(self):
        return self._scheduler

    @scheduler.setter
    def scheduler(self, s: Scheduler):
        self._scheduler = s

    def __str__(self):
        return "{} {}".format(self.timing, self.command)



class LazyProperty:
    def __init__(self, func):
        self._func = func
        self._func_name = self._func.__name__

    def __get__(self, instance, owner):
        instance.__dict__[self._func_name] = self._func(instance)
        return instance.__dict__[self._func_name]



__version__ = "0.0.1"

from cron_migration.cli.manager import CommandsManager
from cron_migration.cli.facade import OperationsFacade

# registering all commands here
import cron_migration.cli.operations.init
import cron_migration.cli.operations.make_revision
import cron_migration.cli.operations.upgrade
import cron_migration.cli.operations.downgrade

# end
from cron_migration.app.models.environment import Environment

environment = Environment()



class Scheduler:
    def __init__(self, time: list = ["*"] * 5):
        self._time = time

    @property
    def time(self):
        return tuple(self._time)

    def every_5_minutes(self):
        self._time[0] = "*/5"
        self._time[1:] = ["*"] * 4

    def every_10_minutes(self):
        self._time[0] = "*/10"
        self._time[1:] = ["*"] * 4

    def every_15_minutes(self):
        self._time[0] = "*/15"
        self._time[1:] = ["*"] * 4

    def every_30_minutes(self):
        self._time[0] = "*/30"
        self._time[1:] = ["*"] * 4

    def once_per_hour(self):
        self._time[0] = "0"
        self._time[1:] = ["*"] * 4

    def at_midnight(self):
        self._time[0:2] = ["0"] * 2
        self._time[2:] = ["*"] * 3
